/***************************************************************************
 *   Copyright (C) 2006 by Michael Andreen                                 *
 *   andreen@student.chalmers.se                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#ifndef __GENEORDER_H__
#define __GENEORDER_H__

#include <vector>
#include <stdexcept>

#include "misc.h"

/**
 * Stores a gene order permutation and ensures that all genes are present 
 * and not duplicated.
 *
 * It has limited support for acting as an STL container, but still maintaining
 * the invariant of a permutation starting with 0 and ending with n, with all
 * genes present.
 *
 * \author Michael Andreen
*/
class GeneOrder{
	public:

		typedef std::vector<Gene> GeneList;
		typedef GeneList::size_type size_type;
		typedef GeneList::const_iterator iterator;

		/**
		 * Creates a copy of the given gene order
		 */
		GeneOrder(const GeneOrder& go);

		/**
		 * Creates a gene order from a given list, using STL-type iterators.
		 *
		 * The given permutation needs to contain all genes 1 to n, if 0 is
		 * included it has to be the first item, if it is not present it will
		 * be added automatically. Similarily, if n is not the last item, then
		 * n+1 will be added to the end.
		 *
		 * \param begin iterator to the first element in the list, from begin()
		 * on stl collections and pointer to first item on plain arrays.
		 *
		 * \param end iterator to the element behind the last, from end() in
		 * stl collections or pointer to first element+size on plain arrays.
		 *
		 * \throws std::invalid_argument if the list is not a valid permutation.
		 */
		template<class T>
		GeneOrder(T begin, T end);

		/**
		 * Destroyes the gene order.
		 */
		~GeneOrder();

		/**
		 * Copies the given gene order.
		 */
		const GeneOrder& operator=(const GeneOrder& go);

		/**
		 * Returns the gene at the given index.
		 *
		 * \throws std::out_of_range if i is smaller than 0 or bigger than n.
		 */
		const Gene& operator[](size_type i) const;

		/**
		 * Returns the size of the permutation.
		 */
		size_type size() const;

		/**
		 * Returns the vector containing the gene order permutation.
		 */
		const GeneList& list() const;

		/**
		 * Returns the start iterator for the permutation.
		 * To be used with STL style functions.
		 *
		 * \see end
		 */
		iterator begin() const;

		/**
		 * Returns the end iterator for the permutation.
		 * To be used with STL style functions.
		 *
		 * \see begin
		 */
		iterator end() const;

		/**
		 * Reverserses the interval [i,j], changing the sign on all elements
		 * affected.
		 *
		 * \throws std::out_of_range if i is smaller than 0 or bigger than n.
		 */
		void reverse(size_type i, size_type j);


	private:
		GeneList _geneorder;
		/**
		 * pads the permutation with 0 in front and n+1 at the end if needed
		 */
		void pad();

		/**
		 * Verifies that the permutation starts with 0, ends with n+1 and that all genes i, 0 < i < n+1
		 * are present, without duplication.
		 */
		void verify();
};


/**
 * Put all the genes in the list, check that all genes are included, pad with 0 and n+1 if needed.
 * TODO: This is far from done atm
 */
template<typename T>
GeneOrder::GeneOrder(T begin, T end): _geneorder(begin,end){
	pad();
	verify();
}

#endif

